@@ -23,7 +23,6 @@
 
 #include "qgis_core.h"
 #include "qgis_sip.h"
-#include "qgis.h"
 #include <QString>
 #include <QFont>
 #include <QFontDatabase>
@@ -80,11 +79,31 @@ class QgsExpressionContext;
 class CORE_EXPORT QgsLabelPosition
 {
   public:
-    QgsLabelPosition( int id, double r, const QVector< QgsPointXY > &corners, const QgsRectangle &rect, double w, double h, const QString &layer, const QString &labeltext, const QFont &labelfont, bool upside_down, bool diagram = false, bool pinned = false, const QString &providerId = QString() )
+
+    /**
+     * Constructor for QgsLabelPosition.
+     * \param id associated feature ID
+     * \param r label rotation in degrees clockwise
+     * \param corners corner points of label bounding box, in map units
+     * \param rect label bounding box, in map units
+     * \param w width of label, in map units
+     * \param h height of label, in map units
+     * \param layer ID of associated map layer
+     * \param labeltext text rendered for label
+     * \param labelfont font used to render label
+     * \param upside_down TRUE if label is upside down
+     * \param diagram TRUE if label is a diagram
+     * \param pinned TRUE if label has pinned placement
+     * \param providerId ID of associated label provider
+     * \param labelGeometry polygon geometry of label boundary
+     */
+    QgsLabelPosition( QgsFeatureId id, double r, const QVector< QgsPointXY > &corners, const QgsRectangle &rect, double w, double h, const QString &layer, const QString &labeltext, const QFont &labelfont, bool upside_down, bool diagram = false, bool pinned = false, const QString &providerId = QString(),
+                      const QgsGeometry &labelGeometry = QgsGeometry() )
       : featureId( id )
       , rotation( r )
       , cornerPoints( corners )
       , labelRect( rect )
+      , labelGeometry( labelGeometry )
       , width( w )
       , height( h )
       , layerID( layer )
@@ -99,19 +118,69 @@ class CORE_EXPORT QgsLabelPosition
     //! Constructor for QgsLabelPosition
     QgsLabelPosition() = default;
 
-    int featureId = -1;
+    /**
+     * ID of feature associated with this label.
+     */
+    QgsFeatureId featureId = FID_NULL;
+
+    /**
+     * Rotation of label, in degrees clockwise.
+     */
     double rotation = 0;
+
     QVector< QgsPointXY > cornerPoints;
     QgsRectangle labelRect;
+
+    /**
+     * A polygon geometry representing the label's bounds in map coordinates.
+     * \since QGIS 3.4.9
+     */
+    QgsGeometry labelGeometry;
+
+    /**
+     * Width of label bounding box, in map units.
+     */
     double width = 0;
+
+    /**
+     * Heeght of label bounding box, in map units.
+     */
     double height = 0;
+
+    /**
+     * ID of associated map layer.
+     */
     QString layerID;
+
+    /**
+     * String shown in label.
+     */
     QString labelText;
+
+    /**
+     * Font which the label is rendered using.
+     */
     QFont labelFont;
+
+    /**
+     * TRUE if label is upside down.
+     */
     bool upsideDown = false;
+
+    /**
+     * TRUE if label is a diagram.
+     */
     bool isDiagram = false;
+
+    /**
+     * TRUE if label position has been pinned.
+     */
     bool isPinned = false;
-    //! \since QGIS 2.14
+
+    /**
+     * ID of the associated label provider.
+     * \since QGIS 2.14
+     */
     QString providerID;
 };
 
@@ -130,56 +199,59 @@ class CORE_EXPORT QgsPalLayerSettings
     //! copy operator - only copies the permanent members
     QgsPalLayerSettings &operator=( const QgsPalLayerSettings &s );
 
+    //TODO QGIS 4.0 - move to QgsLabelingEngine
+
     /**
      * Placement modes which determine how label candidates are generated for a feature.
      */
-    //TODO QGIS 3.0 - move to QgsLabelingEngine
     enum Placement
     {
       AroundPoint, //!< Arranges candidates in a circle around a point (or centroid of a polygon). Applies to point or polygon layers only.
-      OverPoint, //! Arranges candidates over a point (or centroid of a polygon), or at a preset offset from the point. Applies to point or polygon layers only.
+      OverPoint, //!< Arranges candidates over a point (or centroid of a polygon), or at a preset offset from the point. Applies to point or polygon layers only.
       Line, //!< Arranges candidates parallel to a generalised line representing the feature or parallel to a polygon's perimeter. Applies to line or polygon layers only.
-      Curved, //! Arranges candidates following the curvature of a line feature. Applies to line layers only.
+      Curved, //!< Arranges candidates following the curvature of a line feature. Applies to line layers only.
       Horizontal, //!< Arranges horizontal candidates scattered throughout a polygon feature. Applies to polygon layers only.
       Free, //!< Arranges candidates scattered throughout a polygon feature. Candidates are rotated to respect the polygon's orientation. Applies to polygon layers only.
       OrderedPositionsAroundPoint, //!< Candidates are placed in predefined positions around a point. Preference is given to positions with greatest cartographic appeal, e.g., top right, bottom right, etc. Applies to point layers only.
-      PerimeterCurved, //! Arranges candidates following the curvature of a polygon's boundary. Applies to polygon layers only.
+      PerimeterCurved, //!< Arranges candidates following the curvature of a polygon's boundary. Applies to polygon layers only.
     };
 
+    //TODO QGIS 4.0 - move to QgsLabelingEngine
     //! Positions for labels when using the QgsPalLabeling::OrderedPositionsAroundPoint placement mode
-    //TODO QGIS 3.0 - move to QgsLabelingEngine
     enum PredefinedPointPosition
     {
       TopLeft, //!< Label on top-left of point
-      TopSlightlyLeft, //! Label on top of point, slightly left of center
+      TopSlightlyLeft, //!< Label on top of point, slightly left of center
       TopMiddle, //!< Label directly above point
-      TopSlightlyRight, //! Label on top of point, slightly right of center
+      TopSlightlyRight, //!< Label on top of point, slightly right of center
       TopRight, //!< Label on top-right of point
       MiddleLeft, //!< Label on left of point
       MiddleRight, //!< Label on right of point
       BottomLeft, //!< Label on bottom-left of point
-      BottomSlightlyLeft, //! Label below point, slightly left of center
+      BottomSlightlyLeft, //!< Label below point, slightly left of center
       BottomMiddle, //!< Label directly below point
-      BottomSlightlyRight, //! Label below point, slightly right of center
+      BottomSlightlyRight, //!< Label below point, slightly right of center
       BottomRight, //!< Label on bottom right of point
     };
 
+    //TODO QGIS 4.0 - move to QgsLabelingEngine
+
     /**
      * Behavior modifier for label offset and distance, only applies in some
      * label placement modes.
      */
-    //TODO QGIS 3.0 - move to QgsLabelingEngine
     enum OffsetType
     {
       FromPoint, //!< Offset distance applies from point geometry
       FromSymbolBounds, //!< Offset distance applies from rendered symbol bounds
     };
 
+    //TODO QGIS 4.0 - move to QgsLabelingEngine, rename to LinePlacementFlag, use Q_DECLARE_FLAGS to make
+    //LinePlacementFlags type, and replace use of pal::LineArrangementFlag
+
     /**
      * Line placement flags, which control how candidates are generated for a linear feature.
      */
-    //TODO QGIS 3.0 - move to QgsLabelingEngine, rename to LinePlacementFlag, use Q_DECLARE_FLAGS to make
-    //LinePlacementFlags type, and replace use of pal::LineArrangementFlag
     enum LinePlacementFlags
     {
       OnLine    = 1,      //!< Labels can be placed directly over a line feature.
@@ -230,11 +302,12 @@ class CORE_EXPORT QgsPalLayerSettings
                                will be drawn with right alignment*/
     };
 
+    //TODO QGIS 4.0 - Move to QgsLabelingEngine
+
     /**
      * Valid obstacle types, which affect how features within the layer will act as obstacles
      * for labels.
      */
-    //TODO QGIS 3.0 - Move to QgsLabelingEngine
     enum ObstacleType
     {
       PolygonInterior, /*!< avoid placing labels over interior of polygon (prefer placing labels totally
@@ -265,10 +338,11 @@ class CORE_EXPORT QgsPalLayerSettings
       FontCase = 27, //!< Label text case
       FontLetterSpacing = 28, //!< Letter spacing
       FontWordSpacing = 29, //!< Word spacing
-      FontBlendMode = 30, //! Text blend mode
+      FontBlendMode = 30, //!< Text blend mode
 
       // text formatting
       MultiLineWrapChar = 31,
+      AutoWrapLength = 101,
       MultiLineHeight = 32,
       MultiLineAlignment = 33,
       DirSymbDraw = 34,
@@ -347,6 +421,7 @@ class CORE_EXPORT QgsPalLayerSettings
       RepeatDistanceUnit = 86,
       Priority = 87,
       PredefinedPositionOrder = 91,
+      LinePlacementOptions = 99, //!< Line placement flags
 
       // rendering
       ScaleVisibility = 23,
@@ -366,6 +441,20 @@ class CORE_EXPORT QgsPalLayerSettings
       AlwaysShow = 20
     };
 
+
+    /**
+     * Prepare for registration of features.
+     * The \a context, \a mapSettings and \a fields parameters give more
+     * information about the rendering environment.
+     * If target \a crs is not specified, the targetCrs from \a mapSettings
+     * will be taken.
+     * The parameter \a attributeNames should be updated to contain all the field
+     * names which the labeling requires for the rendering.
+     *
+     * \since QGIS 3.8
+     */
+    bool prepare( const QgsRenderContext &context, QSet<QString> &attributeNames SIP_INOUT, const QgsFields &fields, const QgsMapSettings &mapSettings, const QgsCoordinateReferenceSystem &crs );
+
     /**
      * Returns the labeling property definitions.
      * \since QGIS 3.0
@@ -376,7 +465,7 @@ class CORE_EXPORT QgsPalLayerSettings
      * Whether to draw labels for this layer. For some layers it may be desirable
      * to register their features as obstacles for other labels without requiring
      * labels to be drawn for the layer itself. In this case drawLabels can be set
-     * to false and obstacle set to true, which will result in the layer acting
+     * to FALSE and obstacle set to TRUE, which will result in the layer acting
      * as an obstacle but having no labels of its own.
      * \since QGIS 2.12
      */
@@ -386,19 +475,19 @@ class CORE_EXPORT QgsPalLayerSettings
 
     /**
      * Name of field (or an expression) to use for label text.
-     * If fieldName is an expression, then isExpression should be set to true.
+     * If fieldName is an expression, then isExpression should be set to TRUE.
      * \see isExpression
      */
     QString fieldName;
 
     /**
-     * True if this label is made from a expression string, e.g., FieldName || 'mm'
+     * TRUE if this label is made from a expression string, e.g., FieldName || 'mm'
      * \see fieldName
      */
     bool isExpression;
 
     /**
-     * Returns the QgsExpression for this label settings. May be nullptr if isExpression is false.
+     * Returns the QgsExpression for this label settings. May be NULLPTR if isExpression is FALSE.
      */
     QgsExpression *getLabelExpression();
 
@@ -417,11 +506,32 @@ class CORE_EXPORT QgsPalLayerSettings
      */
     QString wrapChar;
 
+    /**
+     * If non-zero, indicates that label text should be automatically wrapped to (ideally) the specified
+     * number of characters. If zero, auto wrapping is disabled.
+     *
+     * \see useMaxLineLengthForAutoWrap
+     * \since QGIS 3.4
+     */
+    int autoWrapLength = 0;
+
+    /**
+     * If TRUE, indicates that when auto wrapping label text the autoWrapLength length indicates the maximum
+     * ideal length of text lines. If FALSE, then autoWrapLength indicates the ideal minimum length of text
+     * lines.
+     *
+     * If autoWrapLength is 0 then this value has no effect.
+     *
+     * \see autoWrapLength
+     * \since QGIS 3.4
+     */
+    bool useMaxLineLengthForAutoWrap = true;
+
     //! Horizontal alignment of multi-line labels.
     MultiLineAlign multilineAlign;
 
     /**
-     * If true, '<' or '>' (or custom strings set via leftDirectionSymbol and rightDirectionSymbol)
+     * If TRUE, '<' or '>' (or custom strings set via leftDirectionSymbol and rightDirectionSymbol)
      * will be automatically added to the label text, pointing in the
      * direction of the line or polygon ring.
      * This setting only affects line or perimeter based labels.
@@ -456,7 +566,7 @@ class CORE_EXPORT QgsPalLayerSettings
     bool reverseDirectionSymbol;
 
     /**
-     * Set to true to format numeric label text as numbers (e.g. inserting thousand separators
+     * Set to TRUE to format numeric label text as numbers (e.g. inserting thousand separators
      * and fixed number of decimal places).
      * \see decimals
      * \see plusSign
@@ -464,14 +574,14 @@ class CORE_EXPORT QgsPalLayerSettings
     bool formatNumbers;
 
     /**
-     * Number of decimal places to show for numeric labels. formatNumbers must be true for this
+     * Number of decimal places to show for numeric labels. formatNumbers must be TRUE for this
      * setting to have an effect.
      * \see formatNumbers
      */
     int decimals;
 
     /**
-     * Whether '+' signs should be prepended to positive numeric labels. formatNumbers must be true for this
+     * Whether '+' signs should be prepended to positive numeric labels. formatNumbers must be TRUE for this
      * setting to have an effect.
      * \see formatNumbers
      */
@@ -483,14 +593,14 @@ class CORE_EXPORT QgsPalLayerSettings
     unsigned int placementFlags;
 
     /**
-     * True if feature centroid should be calculated from the whole feature, or
-     * false if only the visible part of the feature should be considered.
+     * TRUE if feature centroid should be calculated from the whole feature, or
+     * FALSE if only the visible part of the feature should be considered.
      */
     bool centroidWhole;
 
     /**
-     * True if centroid positioned labels must be placed inside their corresponding
-     * feature polygon, or false if centroids which fall outside the polygon
+     * TRUE if centroid positioned labels must be placed inside their corresponding
+     * feature polygon, or FALSE if centroids which fall outside the polygon
      * are permitted.
      */
     bool centroidInside;
@@ -504,7 +614,7 @@ class CORE_EXPORT QgsPalLayerSettings
     QVector< PredefinedPointPosition > predefinedPositionOrder SIP_SKIP;
 
     /**
-     * True if only labels which completely fit within a polygon are allowed.
+     * TRUE if only labels which completely fit within a polygon are allowed.
      */
     bool fitInPolygonOnly;
 
@@ -617,7 +727,7 @@ class CORE_EXPORT QgsPalLayerSettings
     //-- rendering
 
     /**
-     * Set to true to limit label visibility to a range of scales.
+     * Set to TRUE to limit label visibility to a range of scales.
      * \see maximumScale
      * \see minimumScale
      */
@@ -628,7 +738,7 @@ class CORE_EXPORT QgsPalLayerSettings
      * The scale value indicates the scale denominator, e.g. 1000.0 for a 1:1000 map.
      * A scale of 0 indicates no maximum scale visibility.
      *
-     * This setting is only considered if scaleVisibility is true.
+     * This setting is only considered if scaleVisibility is TRUE.
      *
      * \see minimumScale
      * \see scaleVisibility
@@ -640,7 +750,7 @@ class CORE_EXPORT QgsPalLayerSettings
      * The scale value indicates the scale denominator, e.g. 1000.0 for a 1:1000 map.
      * A scale of 0 indicates no minimum scale visibility.
      *
-     * This setting is only considered if scaleVisibility is true.
+     * This setting is only considered if scaleVisibility is TRUE.
      *
      * \see maximumScale
      * \see scaleVisibility
@@ -648,7 +758,7 @@ class CORE_EXPORT QgsPalLayerSettings
     double minimumScale;
 
     /**
-     * True if label sizes should be limited by pixel size.
+     * TRUE if label sizes should be limited by pixel size.
      * \see fontMinPixelSize
      * \see fontMaxPixelSize
      */
@@ -668,33 +778,33 @@ class CORE_EXPORT QgsPalLayerSettings
      */
     int fontMaxPixelSize;
 
-    //! If true, all features will be labelled even when overlaps occur.
+    //! If TRUE, all features will be labelled even when overlaps occur.
     bool displayAll;
 
     //! Controls whether upside down labels are displayed and how they are handled.
     UpsideDownLabels upsidedownLabels = Upright;
 
     /**
-     * True if every part of a multi-part feature should be labeled. If false,
+     * TRUE if every part of a multi-part feature should be labeled. If FALSE,
      * only the largest part will be labeled.
      */
     bool labelPerPart;
 
     /**
-     * True if connected line features with identical label text should be merged
+     * TRUE if connected line features with identical label text should be merged
      * prior to generating label positions.
      */
     bool mergeLines;
 
     /**
-     * True if the number of labels drawn should be limited.
+     * TRUE if the number of labels drawn should be limited.
      * \see maxNumLabels
      */
     bool limitNumLabels;
 
     /**
      * The maximum number of labels which should be drawn for this layer.
-     * This only has an effect if limitNumLabels is true.
+     * This only has an effect if limitNumLabels is TRUE.
      * \see limitNumLabels
      */
     int maxNumLabels;
@@ -705,7 +815,7 @@ class CORE_EXPORT QgsPalLayerSettings
     double minFeatureSize;
 
     /**
-     * True if features for layer are obstacles to labels of other layers.
+     * TRUE if features for layer are obstacles to labels of other layers.
      * \see obstacleFactor
      * \see obstacleType
      */
@@ -729,8 +839,20 @@ class CORE_EXPORT QgsPalLayerSettings
     //! Z-Index of label, where labels with a higher z-index are rendered on top of labels with a lower z-index
     double zIndex;
 
-    // called from register feature hook
-    void calculateLabelSize( const QFontMetricsF *fm, QString text, double &labelX, double &labelY, QgsFeature *f = nullptr, QgsRenderContext *context = nullptr );
+    //! The geometry generator expression. Null if disabled.
+    QString geometryGenerator;
+
+    //! The type of the result geometry of the geometry generator.
+    QgsWkbTypes::GeometryType geometryGeneratorType = QgsWkbTypes::GeometryType::PointGeometry;
+
+    //! Defines if the geometry generator is enabled or not. If disabled, the standard geometry will be taken.
+    bool geometryGeneratorEnabled = false;
+
+    /**
+     * Calculates the space required to render the provided \a text in map units.
+     * Results will be written to \a labelX and \a labelY.
+     */
+    void calculateLabelSize( const QFontMetricsF *fm, const QString &text, double &labelX, double &labelY, const QgsFeature *f = nullptr, QgsRenderContext *context = nullptr );
 
     /**
      * Register a feature for labeling.
@@ -745,7 +867,7 @@ class CORE_EXPORT QgsPalLayerSettings
      * the feature's original geometry will be used as an obstacle for labels. Not available
      * in Python bindings.
      */
-    void registerFeature( QgsFeature &f, QgsRenderContext &context,
+    void registerFeature( const QgsFeature &f, QgsRenderContext &context,
                           QgsLabelFeature **labelFeature SIP_PYARGREMOVE = nullptr,
                           QgsGeometry obstacleGeometry SIP_PYARGREMOVE = QgsGeometry() );
 
@@ -763,24 +885,26 @@ class CORE_EXPORT QgsPalLayerSettings
 
     /**
      * Returns a reference to the label's property collection, used for data defined overrides.
-     * \since QGIS 3.0
      * \see setDataDefinedProperties()
+     * \since QGIS 3.0
      */
     QgsPropertyCollection &dataDefinedProperties() { return mDataDefinedProperties; }
 
     /**
      * Returns a reference to the label's property collection, used for data defined overrides.
-     * \since QGIS 3.0
      * \see setDataDefinedProperties()
+     * \see Property
      * \note not available in Python bindings
+     * \since QGIS 3.0
      */
     const QgsPropertyCollection &dataDefinedProperties() const SIP_SKIP { return mDataDefinedProperties; }
 
     /**
      * Sets the label's property collection, used for data defined overrides.
      * \param collection property collection. Existing properties will be replaced.
-     * \since QGIS 3.0
      * \see dataDefinedProperties()
+     * \see Property
+     * \since QGIS 3.0
      */
     void setDataDefinedProperties( const QgsPropertyCollection &collection ) { mDataDefinedProperties = collection; }
 
@@ -800,7 +924,7 @@ class CORE_EXPORT QgsPalLayerSettings
     void setFormat( const QgsTextFormat &format ) { mFormat = format; }
 
     // temporary stuff: set when layer gets prepared or labeled
-    QgsFeature *mCurFeat = nullptr;
+    const QgsFeature *mCurFeat = nullptr;
     QgsFields mCurFields;
     int fieldIndex;
     const QgsMapToPixel *xform = nullptr;
@@ -812,7 +936,6 @@ class CORE_EXPORT QgsPalLayerSettings
     int mFeaturesToLabel = 0; // total features that will probably be labeled, may be less (figured before PAL)
     int mFeatsSendingToPal = 0; // total features tested for sending into PAL (relative to maxNumLabels)
     int mFeatsRegPal = 0; // number of features registered in PAL, when using limitNumLabels
-
   private:
 
     friend class QgsVectorLayer;  // to allow calling readFromLayerCustomProperties()
@@ -876,7 +999,7 @@ class CORE_EXPORT QgsPalLayerSettings
     /**
      * Registers a feature as an obstacle only (no label rendered)
      */
-    void registerObstacleFeature( QgsFeature &f, QgsRenderContext &context, QgsLabelFeature **obstacleFeature, const QgsGeometry &obstacleGeometry = QgsGeometry() );
+    void registerObstacleFeature( const QgsFeature &f, QgsRenderContext &context, QgsLabelFeature **obstacleFeature, const QgsGeometry &obstacleGeometry = QgsGeometry() );
 
     QMap<Property, QVariant> dataDefinedValues;
 
@@ -889,6 +1012,8 @@ class CORE_EXPORT QgsPalLayerSettings
 
     QgsTextFormat mFormat;
 
+    QgsExpression mGeometryGeneratorExpression;
+
     static const QVector< PredefinedPointPosition > DEFAULT_PLACEMENT_ORDER;
 
     static void initPropertyDefinitions();
@@ -929,17 +1054,23 @@ class CORE_EXPORT QgsLabelingResults
     //! QgsLabelingResults cannot be copied.
     QgsLabelingResults &operator=( const QgsLabelingResults &rh ) = delete;
 
-    //! return infos about labels at a given (map) position
+    //! Returns infos about labels at a given (map) position
     QList<QgsLabelPosition> labelsAtPosition( const QgsPointXY &p ) const;
-    //! return infos about labels within a given (map) rectangle
+    //! Returns infos about labels within a given (map) rectangle
     QList<QgsLabelPosition> labelsWithinRect( const QgsRectangle &r ) const;
 
+    /**
+     * Sets the map \a settings associated with the labeling run.
+     * \since QGIS 3.4.8
+     */
+    void setMapSettings( const QgsMapSettings &settings );
+
   private:
 #ifdef SIP_RUN
     QgsLabelingResults( const QgsLabelingResults & );
 #endif
 
-    QgsLabelSearchTree *mLabelSearchTree = nullptr;
+    std::unique_ptr< QgsLabelSearchTree > mLabelSearchTree;
 
     friend class QgsPalLabeling;
     friend class QgsVectorLayerLabelProvider;
@@ -969,10 +1100,11 @@ class CORE_EXPORT QgsPalLabeling
      * \param context render context
      * \param ct coordinate transform, or invalid transform if no transformation required
      * \param clipGeometry geometry to clip features to, if applicable
+     * \param mergeLines TRUE if touching lines from this layer will be merged and treated as single features during labeling
      * \returns prepared geometry
      * \since QGIS 2.9
      */
-    static QgsGeometry prepareGeometry( const QgsGeometry &geometry, QgsRenderContext &context, const QgsCoordinateTransform &ct, const QgsGeometry &clipGeometry = QgsGeometry() ) SIP_FACTORY;
+    static QgsGeometry prepareGeometry( const QgsGeometry &geometry, QgsRenderContext &context, const QgsCoordinateTransform &ct, const QgsGeometry &clipGeometry = QgsGeometry(), bool mergeLines = false ) SIP_FACTORY;
 
     /**
      * Checks whether a geometry requires preparation before registration with PAL
@@ -980,20 +1112,25 @@ class CORE_EXPORT QgsPalLabeling
      * \param context render context
      * \param ct coordinate transform, or invalid transform if no transformation required
      * \param clipGeometry geometry to clip features to, if applicable
-     * \returns true if geometry requires preparation
+     * \param mergeLines TRUE if touching lines from this layer will be merged and treated as single features during labeling
+     * \returns TRUE if geometry requires preparation
      * \since QGIS 2.9
      */
-    static bool geometryRequiresPreparation( const QgsGeometry &geometry, QgsRenderContext &context, const QgsCoordinateTransform &ct, const QgsGeometry &clipGeometry = QgsGeometry() );
+    static bool geometryRequiresPreparation( const QgsGeometry &geometry, QgsRenderContext &context, const QgsCoordinateTransform &ct, const QgsGeometry &clipGeometry = QgsGeometry(), bool mergeLines = false );
 
     /**
-     * Splits a text string to a list of separate lines, using a specified wrap character.
+     * Splits a \a text string to a list of separate lines, using a specified wrap character (\a wrapCharacter).
      * The text string will be split on either newline characters or the wrap character.
-     * \param text text string to split
-     * \param wrapCharacter additional character to wrap on
-     * \returns list of text split to lines
+     *
+     * Since QGIS 3.4 the \a autoWrapLength argument can be used to specify an ideal length of line to automatically
+     * wrap text to (automatic wrapping is disabled if \a autoWrapLength is 0). This automatic wrapping is performed
+     * after processing wrapping using \a wrapCharacter. When auto wrapping is enabled, the \a useMaxLineLengthWhenAutoWrapping
+     * argument controls whether the lines should be wrapped to an ideal maximum of \a autoWrapLength characters, or
+     * if FALSE then the lines are wrapped to an ideal minimum length of \a autoWrapLength characters.
+     *
      * \since QGIS 2.9
      */
-    static QStringList splitToLines( const QString &text, const QString &wrapCharacter );
+    static QStringList splitToLines( const QString &text, const QString &wrapCharacter, int autoWrapLength = 0, bool useMaxLineLengthWhenAutoWrapping = true );
 
     /**
      * Splits a text string to a list of graphemes, which are the smallest allowable character
@@ -1034,7 +1171,7 @@ class CORE_EXPORT QgsPalLabeling
      * \param context render context
      * \param geom geometry
      * \param minSize minimum size for geometry
-     * \returns true if geometry exceeds minimum size
+     * \returns TRUE if geometry exceeds minimum size
      * \since QGIS 2.9
      */
     static bool checkMinimumSizeMM( const QgsRenderContext &context, const QgsGeometry &geom, double minSize );
